<?php
//	Copyright (c) 2009, 2019 Daniel Robert Bradley. All rights reserved.
//	This software is distributed under the terms of the GNU Lesser General Public License version 2.1
?>
<?php

class Security
{
	static function IsAccessDenied( $pageid )
	{
		$is_denied  = false;
		$restricted = CONTENT_PATH . "/_site/_RESTRICTED.json.txt";

		//
		//	Only denies access if "RESTRICTED" file is present in _site dir.
		//

		if ( file_exists( $restricted ) )
		{
			$json = file_get_contents( $restricted );
			$json = preg_replace( '/\s+/', '', $json );
			$list = json_decode( $json );

			if( ! $list || ! is_array( $list ) )
			{
				$is_denied = true;
				error_log( "ACCESS DENIED: Invalid Configuration: " . $restricted );
			}
			else
			{
				foreach ( $list as $prefix )
				{
					if ( string_startsWith( $pageid, $prefix ) )
					{
						$is_denied = self::VerifyAccessID();
						break;
					}
				}
			}
		}

		if ( ! $is_denied )
		{
			error_log( "ACCESS PERMITTED" );
		}

		return $is_denied;
	}

	static function VerifyAccessID()
	{
		$is_denied = true;
		$status    = "";

		if ( ! array_key_exists( "accessid", $_COOKIE ) )
		{
			$status    = "Missing Access ID";
		}
		else
		{
			$accessid = $_COOKIE["accessid"];

			if ( 0 == strlen( $accessid ) )
			{
				$status = "Empty Access ID";
			}
			else
			if ( $is_denied = self::IsInvalidAccessID( $accessid ) )
			{
				$status = "Invalid";
			}
		}

		if ( $is_denied )
		{
			error_log( "ACCESS DENIED: " . $status );
		}
		return $is_denied;
	}

	static function IsInvalidAccessID( $accessid )
	{
		$is_invalid   = true;
		$api_hostname = self::GenerateAPIHostname( $_SERVER["SERVER_NAME"] );
		$context      = self::CreateStreamContext( $api_hostname, "Aid=$accessid" );

		error_log( "Authenticating using: " . $api_hostname );

		if ( $json = file_get_contents( $api_hostname . "/auth/access/?Aid=$accessid", false, $context ) )
		{
			$obj = json_decode( $json );

			if ( "OK" == $obj->status )
			{
				if ( 1 == count( $obj->results ) )
				{
					if ( "PERMITTED" == $obj->results[0]->access )
					{
						$is_invalid = false;

						define( "CSRF_TOKEN", $obj->results[0]->CSRF_Token );
					}
				}
			}
		}

		if ( $is_invalid ) error_log( $json );

		return $is_invalid;
	}

	static function GenerateAPIHostname( $hostname )
	{
		$port         = (FALSE === strpos( $hostname, ".local" )) ? "443" : "8443";
		$api_hostname = "api-" . $hostname . ":" . $port;

		return ("" !== $_SERVER["HTTPS"]) ? "https://" . $api_hostname : "http://" . $api_hostname;
	}	

	/*
	 *  Copied from BaseSchema Process Places script with permission.
	 *  Copyright 2018 Daniel Bradley
	 */
	function CreateStreamContext( $hostname, $content )
	{
	    $verify_server = (-1 === strpos( $hostname, ".local" ));

	    $NL = "\r\n";

	    $content_type   = "Content-type: application/x-www-form-urlencoded" . $NL;
	    $content_length = "Content-Length: " . strlen( $content )           . $NL;

	    return stream_context_create
	    (
	        array
	        (
	            "http" => array
	            (
	                "method"        => "GET",
	                "header"        => $content_type . $content_length,
	                "content"       => $content,
	                "ignore_errors" => true,
	                "timeout"       => (float)30.0,
	            ),

	            "ssl" => array
	            (
	                "allow_self_signed" => !$verify_server,
	                "verify_peer"       =>  $verify_server,
	                "verify_peer_name"  =>  $verify_server,
	            ),
	        )
	    );
	}
}
